<?php

namespace Drupal\commerce_promotion_by_amount\Plugin\Commerce\PromotionOffer;

use Drupal\commerce_order\Adjustment;
use Drupal\commerce_promotion\Entity\PromotionInterface;
use Drupal\commerce_promotion\Plugin\Commerce\PromotionOffer\FixedAmountOffTrait;
use Drupal\commerce_promotion\Plugin\Commerce\PromotionOffer\OrderItemPromotionOfferBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a fixed amount off by amount matching product in the order.
 *
 * This promotion offer applies a fixed amount discount to the cheapest or most
 * expensive order item (product) in the order that's item is eligible for the
 * promotion.
 *
 * @CommercePromotionOffer(
 *   id = "order_item_fixed_amount_off_by_amount",
 *   label = @Translation("Fixed amount off for cheapest or most expensive matching product"),
 *   entity_type = "commerce_order_item",
 * )
 */
class OrderItemFixedAmountOffByAmount extends OrderItemPromotionOfferBase {

  use FixedAmountOffTrait {
    FixedAmountOffTrait::defaultConfiguration as fixedAmountDefaultConfiguration;
    FixedAmountOffTrait::buildConfigurationForm as fixedAmountBuildConfigurationForm;
    FixedAmountOffTrait::submitConfigurationForm as fixedAmountSubmitConfigurationForm;
  }

  use OrderItemOffByAmountTrait {
    OrderItemOffByAmountTrait::defaultConfiguration as offByAmountDefaultConfiguration;
    OrderItemOffByAmountTrait::buildConfigurationForm as offByAmountBuildConfigurationForm;
    OrderItemOffByAmountTrait::submitConfigurationForm as offByAmountSubmitConfigurationForm;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return $this->offByAmountDefaultConfiguration() + $this->fixedAmountDefaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = $this->fixedAmountBuildConfigurationForm($form, $form_state);
    $form = $this->offByAmountBuildConfigurationForm($form, $form_state);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->fixedAmountSubmitConfigurationForm($form, $form_state);
    $this->offByAmountSubmitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function doApply(EntityInterface $entity, PromotionInterface $promotion) {
    $this->assertEntity($entity);
    /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
    $order_item = $entity;
    $adjusted_total_price = $order_item->getAdjustedTotalPrice(['promotion']);
    $amount = $this->getAmount();
    $multiplier = ($this->configuration['scope'] === static::ORDER_ITEM) ? $order_item->getQuantity() : '1';
    if ($adjusted_total_price->getCurrencyCode() != $amount->getCurrencyCode()) {
      return;
    }
    $adjustment_amount = $amount->multiply($multiplier);
    $adjustment_amount = $this->rounder->round($adjustment_amount);
    if ($adjustment_amount->greaterThan($adjusted_total_price)) {
      // Don't reduce the order item total price past zero.
      $adjustment_amount = $adjusted_total_price;
    }

    // Skip applying the promotion if there's no amount to discount.
    if ($adjustment_amount->isZero()) {
      return;
    }

    $order_item->addAdjustment(new Adjustment([
      'type' => 'promotion',
      'label' => $promotion->getDisplayName() ?: $this->t('Discount'),
      'amount' => $adjustment_amount->multiply('-1'),
      'source_id' => $promotion->id(),
      'included' => $this->configuration['display_inclusive'],
    ]));
  }

}
