<?php

namespace Drupal\commerce_promotion_by_amount\Plugin\Commerce\PromotionOffer;

use Drupal\commerce\ConditionGroup;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_order\Entity\OrderItemInterface;
use Drupal\commerce_price\Calculator;
use Drupal\commerce_promotion\Entity\PromotionInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides common configuration for off by amount offers.
 */
trait OrderItemOffByAmountTrait {

  const CHEAPEST = 'cheapest';
  const MOST_EXPENSIVE = 'most_expensive';

  const PRODUCT = 'product';
  const ORDER_ITEM = 'order_item';

  /**
   * The item the offer should be applied against.
   *
   * @var int|false
   */
  protected int|false $itemToApply;

  /**
   * Applies the offer to the given entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   * @param \Drupal\commerce_promotion\Entity\PromotionInterface $promotion
   *   The parent promotion.
   */
  abstract protected function doApply(EntityInterface $entity, PromotionInterface $promotion);

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'type' => static::CHEAPEST,
      'compare' => static::PRODUCT,
      'scope' => static::ORDER_ITEM,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['type'] = [
      '#type' => 'radios',
      '#title' => $this->t("The promotion will affect the cheapest or the most expensive item?"),
      '#options' => [
        static::CHEAPEST => $this->t('Cheapest item'),
        static::MOST_EXPENSIVE => $this->t('Most expensive item'),
      ],
      '#default_value' => $this->configuration['type'],
      '#weight' => -1,
    ];

    $form['compare'] = [
      '#type' => 'radios',
      '#title' => $this->t("The promotion will compare the order item or the product amount?"),
      '#options' => [
        static::PRODUCT => $this->t('Product amount'),
        static::ORDER_ITEM => $this->t('Order Item amount'),
      ],
      '#default_value' => $this->configuration['compare'],
      '#weight' => -1,
    ];

    $form['scope'] = [
      '#type' => 'radios',
      '#title' => $this->t("The promotion will apply to all the products in the order item or only one?"),
      '#options' => [
        static::ORDER_ITEM => $this->t('All the products'),
        static::PRODUCT => $this->t('Only one product'),
      ],
      '#default_value' => $this->configuration['scope'],
      '#weight' => -1,
    ];

    if (isset($form['display_inclusive'])) {
      $form['display_inclusive']['#access'] = FALSE;
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['type'] = $values['type'];
      $this->configuration['compare'] = $values['compare'];
      $this->configuration['scope'] = $values['scope'];
      $this->configuration['display_inclusive'] = FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function apply(EntityInterface $entity, PromotionInterface $promotion) {
    /** @var \Drupal\commerce_order\Entity\OrderItemInterface $entity */
    $this->assertEntity($entity);

    $selected_item = $this->getItemToApply($entity, $promotion);

    if ($selected_item == $entity->id()) {
      $this->doApply($entity, $promotion);
    }

  }

  /**
   * Sorts the order item by total price asc or desc based on the config.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   The order.
   *
   * @return array
   *   Array of order items sorted by total price.
   */
  protected function sortItems(OrderInterface $order): array {
    $items = $order->getItems();

    $compare = ($this->configuration['compare'] === self::ORDER_ITEM) ?
      fn (OrderItemInterface $item1, OrderItemInterface $item2) =>
      $item1->getTotalPrice()->compareTo($item2->getTotalPrice()) :
      fn (OrderItemInterface $item1, OrderItemInterface $item2) =>
      $item1->getUnitPrice()->compareTo($item2->getUnitPrice());

    usort($items, $compare);

    return $this->configuration['type'] === self::CHEAPEST ? $items : array_reverse($items);
  }

  /**
   * Gets the order item the offer applies to. FALSE if none of them applies.
   *
   * @param \Drupal\commerce_order\Entity\OrderItemInterface $order_item
   *   The order item.
   * @param \Drupal\commerce_promotion\Entity\PromotionInterface $promotion
   *   The promotion.
   *
   * @return int|false
   *   The item to apply ID if there is any, NULL otherwise
   */
  protected function getItemToApply(OrderItemInterface $order_item, PromotionInterface $promotion): int|false {
    if (isset($this->itemToApply)) {
      return $this->itemToApply;
    }

    $order = $order_item->getOrder();
    $sorted_items = $this->sortItems($order);

    $offer_conditions = new ConditionGroup($this->getConditions(), $this->getConditionOperator());
    // Apply the offer to order items that pass the conditions.
    foreach ($sorted_items as $item) {
      // Skip order items with a null unit price or with a quantity = 0.
      if (!$item->getUnitPrice() || Calculator::compare($order_item->getQuantity(), '0') === 0) {
        continue;
      }
      if ($offer_conditions->evaluate($item)) {
        $this->itemToApply = $item->id();
        return $this->itemToApply;
      }
    }

    return FALSE;
  }

}
